@@ -3,6 +3,9 @@
 #include <pybind11/stl.h>
 #include "gmmtree.h"
 
+#define STRINGIFY(x) #x
+#define MACRO_STRINGIFY(x) STRINGIFY(x)
+
 namespace py = pybind11;
 using namespace probreg;
 
@@ -13,7 +16,7 @@ PYBIND11_MODULE(_gmmtree, m) {
     m.def("gmmtree_reg_estep", gmmTreeRegEstep);
 
 #ifdef VERSION_INFO
-    m.attr("__version__") = VERSION_INFO;
+    m.attr("__version__") = MACRO_STRINGIFY(VERSION_INFO);
 #else
     m.attr("__version__") = "dev";
 #endif
