@@ -2,6 +2,9 @@
 #include <pybind11/pybind11.h>
 #include "ifgt.h"
 
+#define STRINGIFY(x) #x
+#define MACRO_STRINGIFY(x) STRINGIFY(x)
+
 namespace py = pybind11;
 using namespace probreg;
 
@@ -16,7 +19,7 @@ PYBIND11_MODULE(_ifgt, m) {
     });
 
 #ifdef VERSION_INFO
-    m.attr("__version__") = VERSION_INFO;
+    m.attr("__version__") = MACRO_STRINGIFY(VERSION_INFO);
 #else
     m.attr("__version__") = "dev";
 #endif
