@@ -3,6 +3,9 @@
 #include <pybind11/pybind11.h>
 #include <Eigen/Geometry>
 
+#define STRINGIFY(x) #x
+#define MACRO_STRINGIFY(x) STRINGIFY(x)
+
 using namespace probreg;
 namespace py = pybind11;
 
@@ -13,7 +16,7 @@ PYBIND11_MODULE(_kabsch, m) {
     m.def("kabsch2d", &computeKabsch2d);
 
 #ifdef VERSION_INFO
-    m.attr("__version__") = VERSION_INFO;
+    m.attr("__version__") = MACRO_STRINGIFY(VERSION_INFO);
 #else
     m.attr("__version__") = "dev";
 #endif
