@@ -2,6 +2,9 @@
 #include <pybind11/pybind11.h>
 #include "math_utils.h"
 
+#define STRINGIFY(x) #x
+#define MACRO_STRINGIFY(x) STRINGIFY(x)
+
 namespace py = pybind11;
 using namespace probreg;
 
@@ -15,7 +18,7 @@ PYBIND11_MODULE(_math, m) {
     m.def("inverse_multiquadric_kernel", &inverseMultiQuadricKernel);
 
 #ifdef VERSION_INFO
-    m.attr("__version__") = VERSION_INFO;
+    m.attr("__version__") = MACRO_STRINGIFY(VERSION_INFO);
 #else
     m.attr("__version__") = "dev";
 #endif
