@@ -3,6 +3,9 @@
 #include "permutohedral.h"
 #include "types.h"
 
+#define STRINGIFY(x) #x
+#define MACRO_STRINGIFY(x) STRINGIFY(x)
+
 namespace py = pybind11;
 using namespace probreg;
 
@@ -27,7 +30,7 @@ PYBIND11_MODULE(_permutohedral_lattice, m) {
     });
 
 #ifdef VERSION_INFO
-    m.attr("__version__") = VERSION_INFO;
+    m.attr("__version__") = MACRO_STRINGIFY(VERSION_INFO);
 #else
     m.attr("__version__") = "dev";
 #endif
