@@ -11,6 +11,9 @@ Pt2PlResult probreg::computeTwistForPointToPlane(const MatrixX3& model,
     Vector6 atb = Vector6::Zero();
     Float r_sum = 0.0;
 
+    #pragma omp declare reduction(+ : Matrix6 : omp_out=omp_out+omp_in) initializer(omp_priv = omp_orig)
+    #pragma omp declare reduction(+ : Vector6 : omp_out=omp_out+omp_in) initializer(omp_priv = omp_orig)
+    #pragma omp parallel for reduction(+:ata) reduction(+:atb) reduction(+:r_sum)
     for (auto k = 0; k < model.rows(); ++k){
         const auto& vertex_k = model.row(k).transpose();
         const auto& target_k = target.row(k).transpose();
@@ -18,9 +21,12 @@ Pt2PlResult probreg::computeTwistForPointToPlane(const MatrixX3& model,
         const auto& weight_k = weight[k];
         const Float residual = normal_k.dot(target_k - vertex_k);
         const Vector6 jac = (Vector6() << vertex_k.cross(normal_k), normal_k).finished();
-        ata.noalias() += weight_k * jac * jac.transpose();
-        atb.noalias() += weight_k * residual * jac;
-        r_sum += weight_k * weight_k * residual * residual;
+        const Matrix6 wjjt = weight_k * jac * jac.transpose();
+        const Vector6 wrj = weight_k * residual * jac;
+        const Float wr = weight_k * weight_k * residual * residual;
+        ata.noalias() += wjjt;
+        atb.noalias() += wrj;
+        r_sum += wr;
     }
     return std::make_pair(ata.selfadjointView<Eigen::Upper>().ldlt().solve(atb), r_sum);
 }
