@@ -1,21 +1,24 @@
-from __future__ import print_function
-from __future__ import division
+from __future__ import division, print_function
+from types import ModuleType
+from typing import Any, Callable, Dict, List, Optional, Union
+
 import abc
 from collections import namedtuple
-import six
+
 import numpy as np
 import open3d as o3
-from . import transformation as tf
+import six
+
 from . import math_utils as mu
+from . import transformation as tf
 from .log import log
 
-
-EstepResult = namedtuple('EstepResult', ['pt1', 'p1', 'px', 'n_p'])
-MstepResult = namedtuple('MstepResult', ['transformation', 'sigma2', 'q'])
+EstepResult = namedtuple("EstepResult", ["pt1", "p1", "px", "n_p"])
+MstepResult = namedtuple("MstepResult", ["transformation", "sigma2", "q"])
 
 
 @six.add_metaclass(abc.ABCMeta)
-class CoherentPointDrift():
+class CoherentPointDrift:
     """Coherent Point Drift algorithm.
     This is an abstract class.
     Based on this class, it is inherited by rigid, affine, nonrigid classes
@@ -27,13 +30,15 @@ class CoherentPointDrift():
         source (numpy.ndarray, optional): Source point cloud data.
         use_cuda (bool, optional): Use CUDA.
     """
-    def __init__(self, source=None, use_cuda=False):
+
+    def __init__(self, source: Optional[np.ndarray] = None, use_cuda: bool = False) -> None:
         self._source = source
         self._tf_type = None
         self._callbacks = []
         if use_cuda:
             import cupy as cp
             from . import cupy_utils
+
             self.xp = cp
             self.cupy_utils = cupy_utils
             self._squared_kernel_sum = cupy_utils.squared_kernel_sum
@@ -41,19 +46,20 @@ class CoherentPointDrift():
             self.xp = np
             self._squared_kernel_sum = mu.squared_kernel_sum
 
-    def set_source(self, source):
+    def set_source(self, source: np.ndarray) -> None:
         self._source = source
 
-    def set_callbacks(self, callbacks):
+    def set_callbacks(self, callbacks: List[Callable]) -> None:
         self._callbacks.extend(callbacks)
 
     @abc.abstractmethod
-    def _initialize(self, target):
+    def _initialize(self, target: np.ndarray) -> MstepResult:
         return MstepResult(None, None, None)
 
-    def expectation_step(self, t_source, target, sigma2, w=0.0):
-        """Expectation step for CPD
-        """
+    def expectation_step(
+        self, t_source: np.ndarray, target: np.ndarray, sigma2: float, w: float = 0.0
+    ) -> EstepResult:
+        """Expectation step for CPD"""
         assert t_source.ndim == 2 and target.ndim == 2, "source and target must have 2 dimensions."
         pmat = self.xp.stack([self.xp.sum(self.xp.square(target - ts), axis=1) for ts in t_source])
         pmat = self.xp.exp(-pmat / (2.0 * sigma2))
@@ -61,25 +67,30 @@ class CoherentPointDrift():
         c = (2.0 * np.pi * sigma2) ** (t_source.shape[1] * 0.5)
         c *= w / (1.0 - w) * t_source.shape[0] / target.shape[0]
         den = self.xp.sum(pmat, axis=0)
-        den[den==0] = self.xp.finfo(np.float32).eps
+        den[den == 0] = self.xp.finfo(np.float32).eps
         den += c
 
-        pmat  = self.xp.divide(pmat, den)
+        pmat = self.xp.divide(pmat, den)
         pt1 = self.xp.sum(pmat, axis=0)
-        p1  = self.xp.sum(pmat, axis=1)
+        p1 = self.xp.sum(pmat, axis=1)
         px = self.xp.dot(pmat, target)
         return EstepResult(pt1, p1, px, np.sum(p1))
 
-    def maximization_step(self, target, estep_res, sigma2_p=None):
+    def maximization_step(
+        self, target: np.ndarray, estep_res: EstepResult, sigma2_p: Optional[float] = None
+    ) -> Optional[MstepResult]:
         return self._maximization_step(self._source, target, estep_res, sigma2_p, xp=self.xp)
 
     @staticmethod
     @abc.abstractmethod
-    def _maximization_step(source, target, estep_res, sigma2_p=None, xp=np):
+    def _maximization_step(
+        source: np.ndarray, target: np.ndarray, estep_res: EstepResult, sigma2_p: Optional[float] = None, xp: ModuleType = np
+    ) -> Optional[MstepResult]:
         return None
 
-    def registration(self, target, w=0.0,
-                     maxiter=50, tol=0.001):
+    def registration(
+        self, target: np.ndarray, w: float = 0.0, maxiter: int = 50, tol: float = 0.001
+    ) -> MstepResult:
         assert not self._tf_type is None, "transformation type is None."
         res = self._initialize(target)
         q = res.q
@@ -105,14 +116,16 @@ class RigidCPD(CoherentPointDrift):
         tf_init_params (dict, optional): Parameters to initialize transformation.
         use_cuda (bool, optional): Use CUDA.
     """
-    def __init__(self, source=None, update_scale=True,
-                 tf_init_params={}, use_cuda=False):
+
+    def __init__(
+        self, source: Optional[np.ndarray] = None, update_scale: bool = True, tf_init_params: Dict = {}, use_cuda: bool = False
+    ) -> None:
         super(RigidCPD, self).__init__(source, use_cuda)
         self._tf_type = tf.RigidTransformation
         self._update_scale = update_scale
         self._tf_init_params = tf_init_params
 
-    def _initialize(self, target):
+    def _initialize(self, target: np.ndarray) -> MstepResult:
         dim = self._source.shape[1]
         sigma2 = self._squared_kernel_sum(self._source, target)
         q = 1.0 + target.shape[0] * dim * 0.5 * np.log(sigma2)
@@ -122,13 +135,15 @@ class RigidCPD(CoherentPointDrift):
             self._tf_init_params["xp"] = self.xp
         return MstepResult(self._tf_type(**self._tf_init_params), sigma2, q)
 
-    def maximization_step(self, target, estep_res, sigma2_p=None):
-        return self._maximization_step(self._source, target, estep_res,
-                                       sigma2_p, self._update_scale, self.xp)
+    def maximization_step(
+        self, target: np.ndarray, estep_res: EstepResult, sigma2_p: Optional[float] = None
+    ) -> MstepResult:
+        return self._maximization_step(self._source, target, estep_res, sigma2_p, self._update_scale, self.xp)
 
     @staticmethod
-    def _maximization_step(source, target, estep_res,
-                           sigma2_p=None, update_scale=True, xp=np):
+    def _maximization_step(
+        source: np.ndarray, target: np.ndarray, estep_res: EstepResult, sigma2_p: Optional[float] = None, update_scale: bool = True, xp: ModuleType = np
+    ) -> MstepResult:
         pt1, p1, px, n_p = estep_res
         dim = source.shape[1]
         mu_x = xp.sum(px, axis=0) / n_p
@@ -163,12 +178,13 @@ class AffineCPD(CoherentPointDrift):
         tf_init_params (dict, optional): Parameters to initialize transformation.
         use_cuda (bool, optional): Use CUDA.
     """
-    def __init__(self, source=None, tf_init_params={}, use_cuda=False):
+
+    def __init__(self, source: Optional[np.ndarray] = None, tf_init_params: Dict = {}, use_cuda: bool = False) -> None:
         super(AffineCPD, self).__init__(source, use_cuda)
         self._tf_type = tf.AffineTransformation
         self._tf_init_params = tf_init_params
 
-    def _initialize(self, target):
+    def _initialize(self, target: np.ndarray) -> MstepResult:
         dim = self._source.shape[1]
         sigma2 = self._squared_kernel_sum(self._source, target)
         q = 1.0 + target.shape[0] * dim * 0.5 * np.log(sigma2)
@@ -179,7 +195,9 @@ class AffineCPD(CoherentPointDrift):
         return MstepResult(self._tf_type(**self._tf_init_params), sigma2, q)
 
     @staticmethod
-    def _maximization_step(source, target, estep_res, sigma2_p=None, xp=np):
+    def _maximization_step(
+        source: np.ndarray, target: np.ndarray, estep_res: EstepResult, sigma2_p: Optional[float] = None, xp: ModuleType = np
+    ) -> MstepResult:
         pt1, p1, px, n_p = estep_res
         dim = source.shape[1]
         mu_x = xp.sum(px, axis=0) / n_p
@@ -209,7 +227,10 @@ class NonRigidCPD(CoherentPointDrift):
         lmd (float, optional): Parameter for regularization term.
         use_cuda (bool, optional): Use CUDA.
     """
-    def __init__(self, source=None, beta=2.0, lmd=2.0, use_cuda=False):
+
+    def __init__(
+        self, source: Optional[np.ndarray] = None, beta: float = 2.0, lmd: float = 2.0, use_cuda: bool = False
+    ) -> None:
         super(NonRigidCPD, self).__init__(source, use_cuda)
         self._tf_type = tf.NonRigidTransformation
         self._beta = beta
@@ -218,15 +239,14 @@ class NonRigidCPD(CoherentPointDrift):
         if not self._source is None:
             self._tf_obj = self._tf_type(None, self._source, self._beta, self.xp)
 
-    def set_source(self, source):
+    def set_source(self, source: np.ndarray) -> None:
         self._source = source
         self._tf_obj = self._tf_type(None, self._source, self._beta)
 
-    def maximization_step(self, target, estep_res, sigma2_p=None):
-        return self._maximization_step(self._source, target, estep_res,
-                                       sigma2_p, self._tf_obj, self._lmd, self.xp)
+    def maximization_step(self, target: np.ndarray, estep_res: EstepResult, sigma2_p: Optional[float] = None) -> MstepResult:
+        return self._maximization_step(self._source, target, estep_res, sigma2_p, self._tf_obj, self._lmd, self.xp)
 
-    def _initialize(self, target):
+    def _initialize(self, target: np.ndarray) -> MstepResult:
         dim = self._source.shape[1]
         sigma2 = self._squared_kernel_sum(self._source, target)
         q = 1.0 + target.shape[0] * dim * 0.5 * np.log(sigma2)
@@ -234,11 +254,12 @@ class NonRigidCPD(CoherentPointDrift):
         return MstepResult(self._tf_obj, sigma2, q)
 
     @staticmethod
-    def _maximization_step(source, target, estep_res, sigma2_p, tf_obj, lmd, xp=np):
+    def _maximization_step(
+        source: np.ndarray, target: np.ndarray, estep_res: EstepResult, sigma2_p: float, tf_obj: tf.NonRigidTransformation, lmd: float, xp: ModuleType = np
+    ) -> MstepResult:
         pt1, p1, px, n_p = estep_res
         dim = source.shape[1]
-        w = xp.linalg.solve((p1 * tf_obj.g).T + lmd * sigma2_p * xp.identity(source.shape[0]),
-                            px - (source.T * p1).T)
+        w = xp.linalg.solve((p1 * tf_obj.g).T + lmd * sigma2_p * xp.identity(source.shape[0]), px - (source.T * p1).T)
         t = source + xp.dot(tf_obj.g, w)
         tr_xp1x = xp.trace(xp.dot(target.T * pt1, target))
         tr_pxt = xp.trace(xp.dot(px.T, t))
@@ -248,35 +269,119 @@ class NonRigidCPD(CoherentPointDrift):
         return MstepResult(tf_obj, sigma2, sigma2)
 
 
-def registration_cpd(source, target, tf_type_name='rigid',
-                     w=0.0, maxiter=50, tol=0.001,
-                     callbacks=[], **kargs):
+class ConstrainedNonRigidCPD(CoherentPointDrift):
+    """
+       Extended Coherent Point Drift for nonrigid transformation.
+       Like CoherentPointDrift, but allows to add point correspondance constraints
+       See: https://people.mpi-inf.mpg.de/~golyanik/04_DRAFTS/ECPD2016.pdf
+
+    Args:
+        source (numpy.ndarray, optional): Source point cloud data.
+        beta (float, optional): Parameter of RBF kernel.
+        lmd (float, optional): Parameter for regularization term.
+        alpha (float): Degree of reliability of priors.
+            Approximately between 1e-8 (highly reliable) and 1 (highly unreliable)
+        use_cuda (bool, optional): Use CUDA.
+        idx_source (numpy.ndarray of ints, optional): Indices in source matrix
+            for which a correspondance is known
+        idx_target (numpy.ndarray of ints, optional): Indices in target matrix
+            for which a correspondance is known
+    """
+    def __init__(self, source: Optional[np.ndarray] = None, beta: float = 2.0, lmd: float = 2.0, alpha: float = 1e-8, use_cuda: bool = False, 
+                 idx_source: Optional[np.ndarray] = None, idx_target: Optional[np.ndarray] = None):
+        super(ConstrainedNonRigidCPD, self).__init__(source, use_cuda)
+        self._tf_type = tf.NonRigidTransformation
+        self._beta = beta
+        self._lmd = lmd
+        self.alpha= alpha
+        self._tf_obj = None
+        self.idx_source, self.idx_target= idx_source, idx_target
+        if not self._source is None:
+            self._tf_obj = self._tf_type(None, self._source, self._beta, self.xp)
+
+    def set_source(self, source: np.ndarray) -> None:
+        self._source = source
+        self._tf_obj = self._tf_type(None, self._source, self._beta)
+
+    def maximization_step(self, target: np.ndarray, estep_res: EstepResult, sigma2_p: Optional[float] = None) -> MstepResult:
+        return self._maximization_step(self._source, target, estep_res,
+                   sigma2_p, self._tf_obj, self._lmd,
+                   self.alpha, self.p1_tilde, self.px_tilde, self.xp)
+
+    def _initialize(self, target: np.ndarray) -> MstepResult:
+        dim = self._source.shape[1]
+        sigma2 = self._squared_kernel_sum(self._source, target)
+        q = 1.0 + target.shape[0] * dim * 0.5 * np.log(sigma2)
+        self._tf_obj.w = self.xp.zeros_like(self._source)
+        self.p_tilde= self.xp.zeros((self._source.shape[0], target.shape[0]))
+        if self.idx_source is not None and self.idx_target is not None:
+            self.p_tilde[self.idx_source, self.idx_target]= 1
+        self.p1_tilde = self.xp.sum(self.p_tilde, axis=1)
+        self.px_tilde = self.xp.dot(self.p_tilde, target)
+        return MstepResult(self._tf_obj, sigma2, q)
+
+    @staticmethod
+    def _maximization_step(source: np.ndarray, target: np.ndarray, estep_res: EstepResult, sigma2_p: float, tf_obj: tf.NonRigidTransformation, lmd: float, 
+                           alpha: float, p1_tilde: float, px_tilde: float, xp: ModuleType = np) -> MstepResult:
+        pt1, p1, px, n_p = estep_res
+        dim = source.shape[1]
+        w = xp.linalg.solve( (p1 * tf_obj.g).T + sigma2_p / alpha * (p1_tilde * tf_obj.g).T \
+                                               + lmd * sigma2_p * xp.identity(source.shape[0]),
+                             px - (source.T * p1).T + sigma2_p / alpha * ( px_tilde - (source.T * p1_tilde).T )
+                            )
+        t = source + xp.dot(tf_obj.g, w)
+        tr_xp1x = xp.trace(xp.dot(target.T * pt1, target))
+        tr_pxt = xp.trace(xp.dot(px.T, t))
+        tr_tpt = xp.trace(xp.dot(t.T * p1, t))
+        sigma2 = (tr_xp1x - 2.0 * tr_pxt + tr_tpt) / (n_p * dim)
+        tf_obj.w = w
+        return MstepResult(tf_obj, sigma2, sigma2)
+
+
+def registration_cpd(
+    source: Union[np.ndarray, o3.geometry.PointCloud],
+    target: Union[np.ndarray, o3.geometry.PointCloud],
+    tf_type_name: str = "rigid",
+    w: float = 0.0,
+    maxiter: int = 50,
+    tol: float = 0.001,
+    callbacks: List[Callable] = [],
+    use_cuda: bool = False,
+    **kwargs: Any,
+):
     """CPD Registraion.
 
     Args:
         source (numpy.ndarray): Source point cloud data.
         target (numpy.ndarray): Target point cloud data.
-        tf_type_name (str, optional): Transformation type('rigid', 'affine', 'nonrigid')
+        tf_type_name (str, optional): Transformation type('rigid', 'affine', 'nonrigid', 'nonrigid_constrained')
         w (float, optional): Weight of the uniform distribution, 0 < `w` < 1.
         maxitr (int, optional): Maximum number of iterations to EM algorithm.
         tol (float, optional): Tolerance for termination.
         callback (:obj:`list` of :obj:`function`, optional): Called after each iteration.
             `callback(probreg.Transformation)`
+        use_cuda (bool, optional): Use CUDA.
 
     Keyword Args:
         update_scale (bool, optional): If this flag is true and tf_type is rigid transformation,
             then the scale is treated. The default is true.
         tf_init_params (dict, optional): Parameters to initialize transformation (for rigid or affine).
     """
-    cv = lambda x: np.asarray(x.points if isinstance(x, o3.geometry.PointCloud) else x)
-    if tf_type_name == 'rigid':
-        cpd = RigidCPD(cv(source), **kargs)
-    elif tf_type_name == 'affine':
-        cpd = AffineCPD(cv(source), **kargs)
-    elif tf_type_name == 'nonrigid':
-        cpd = NonRigidCPD(cv(source), **kargs)
+    xp = np
+    if use_cuda:
+        import cupy as cp
+
+        xp = cp
+    cv = lambda x: xp.asarray(x.points if isinstance(x, o3.geometry.PointCloud) else x)
+    if tf_type_name == "rigid":
+        cpd = RigidCPD(cv(source), use_cuda=use_cuda, **kwargs)
+    elif tf_type_name == "affine":
+        cpd = AffineCPD(cv(source), use_cuda=use_cuda, **kwargs)
+    elif tf_type_name == "nonrigid":
+        cpd = NonRigidCPD(cv(source), use_cuda=use_cuda, **kwargs)
+    elif tf_type_name == 'nonrigid_constrained':
+        cpd = ConstrainedNonRigidCPD(cv(source), use_cuda=use_cuda, **kwargs)
     else:
-        raise ValueError('Unknown transformation type %s' % tf_type_name)
+        raise ValueError("Unknown transformation type %s" % tf_type_name)
     cpd.set_callbacks(callbacks)
-    return cpd.registration(cv(target),
-                            w, maxiter, tol)
+    return cpd.registration(cv(target), w, maxiter, tol)
