@@ -1,6 +1,7 @@
-from __future__ import print_function
-from __future__ import division
+from __future__ import division, print_function
+
 import numpy as np
+
 from . import _permutohedral_lattice
 
 
