@@ -1,5 +1,6 @@
 import logging
-log = logging.getLogger('probreg')
+
+log = logging.getLogger("probreg")
 log.setLevel(logging.INFO)
 ch = logging.StreamHandler()
-log.addHandler(ch)
\ No newline at end of file
+log.addHandler(ch)
